/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id:
 */
package edu.indiana.extreme.xbaya;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.List;

import xsul5.MLogger;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.gui.ComponentSelector;
import edu.indiana.extreme.xbaya.component.gui.ComponentTreeNode;
import edu.indiana.extreme.xbaya.component.registry.ComponentRegistryException;
import edu.indiana.extreme.xbaya.component.registry.LocalComponentRegistry;
import edu.indiana.extreme.xbaya.component.registry.SystemComponentRegistry;
import edu.indiana.extreme.xbaya.gpel.GPELClient;
import edu.indiana.extreme.xbaya.gpel.GPELException;
import edu.indiana.extreme.xbaya.gpel.GPELClient.WorkflowType;
import edu.indiana.extreme.xbaya.gpel.component.SubWorkflowUpdater;
import edu.indiana.extreme.xbaya.gpel.gui.GPELLoader;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;
import edu.indiana.extreme.xbaya.gui.ErrorWindow;
import edu.indiana.extreme.xbaya.gui.XBayaGUI;
import edu.indiana.extreme.xbaya.monitor.Monitor;
import edu.indiana.extreme.xbaya.monitor.MonitorConfiguration;
import edu.indiana.extreme.xbaya.monitor.gui.MonitorStarter;
import edu.indiana.extreme.xbaya.mylead.MyLead;
import edu.indiana.extreme.xbaya.mylead.MyLeadConfiguration;
import edu.indiana.extreme.xbaya.mylead.gui.MyLeadLoader;
import edu.indiana.extreme.xbaya.myproxy.MyProxyClient;
import edu.indiana.extreme.xbaya.myproxy.gui.MyProxyDialog;
import edu.indiana.extreme.xbaya.rescat.ResourceCatalogClient;
import edu.indiana.extreme.xbaya.security.XBayaSecurity;
import edu.indiana.extreme.xbaya.util.XMLUtil;
import edu.indiana.extreme.xbaya.wf.Workflow;
import edu.indiana.extreme.xbaya.xregistry.XRegistry;

/**
 * @author Satoshi Shirasuna
 */
public class XBayaEngine {

    private static final MLogger logger = MLogger.getLogger();

    private XBayaConfiguration configuration;

    private XBayaGUI gui;

    private MyLead myLead;

    private GPELClient gpelClient;

    private SubWorkflowUpdater subWorkflowUpdater;

    private Monitor monitor;

    private MyProxyClient myProxyClient;

    /**
     * Constructs a ApplicationClient.
     * 
     * @param configuration
     */
    public XBayaEngine(XBayaConfiguration configuration) {
        this.configuration = configuration;

        // Creates some essential objects.

        MonitorConfiguration monitorConfiguration = new MonitorConfiguration(
                configuration.getBrokerURL(), configuration.getTopic(),
                configuration.isPullMode(), configuration.getMessageBoxURL());
        this.monitor = new Monitor(monitorConfiguration);

        // MyProxy
        this.myProxyClient = new MyProxyClient(this.configuration
                .getMyProxyServer(), this.configuration.getMyProxyPort(),
                this.configuration.getMyProxyUsername(), this.configuration
                        .getMyProxyPassphrase(), this.configuration
                        .getMyProxyLifetime());

        // These have to be before the GUI setup.
        this.gpelClient = new GPELClient();
        this.myLead = new MyLead();

        this.subWorkflowUpdater = new SubWorkflowUpdater(this);

        // Set up the GUI.
        XBayaEngine.this.gui = new XBayaGUI(XBayaEngine.this);

        // Arguments errors.
        for (Throwable e : this.configuration.getErrors()) {
            getErrorWindow().error(e.getMessage(), e);
        }

        // Initialization after the GUI setup.
        initAfterGUI();
    }

    /**
     * Returns the configuration.
     * 
     * @return The configuration
     */
    public XBayaConfiguration getConfiguration() {
        return this.configuration;
    }

    /**
     * Returns the XwfGui. The XwfGui is either XwfAppletGui in case of the
     * applet, or XwfApplicationGui in case of the application.
     * 
     * @return the XwfGui
     */
    public XBayaGUI getGUI() {
        return this.gui;
    }

    /**
     * Returns the GPEL Client.
     * 
     * @return the GPEL Client
     */
    public GPELClient getGPELClient() {
        return this.gpelClient;
    }

    /**
     * Returns the subWorkflowUpdater.
     * 
     * @return The subWorkflowUpdater
     */
    public SubWorkflowUpdater getSubWorkflowUpdater() {
        return this.subWorkflowUpdater;
    }

    /**
     * Returns the MyLeadConnection.
     * 
     * @return the MyLeadConnection
     */
    public MyLead getMyLead() {
        return this.myLead;
    }

    /**
     * Sets the workflow.
     * 
     * @param workflow
     *            The workflow
     */
    public void setWorkflow(Workflow workflow) {
        this.gui.getGraphCanvas().setWorkflow(workflow);
    }

    /**
     * Return the current workflow.
     * 
     * @return The current workflow
     */
    public Workflow getWorkflow() {
        return this.gui.getGraphCanvas().getWorkflowWithImage();
    }

    /**
     * Returns the ErrorWindow. The ErrorWindow is used to show error messages.
     * 
     * @return the ErrorWindow
     */
    public ErrorWindow getErrorWindow() {
        return this.gui.getErrorWindow();
    }

    /**
     * Returns the monitor.
     * 
     * @return The monitor
     */
    public Monitor getMonitor() {
        return this.monitor;
    }

    /**
     * @return The MyProxyClient.
     */
    public MyProxyClient getMyProxyClient() {
        return this.myProxyClient;
    }

    /**
     * Disposes on exit.
     * 
     * @throws XBayaException
     */
    public void dispose() throws XBayaException {
        this.monitor.stop();
    }

    /**
     * Initialization process. This method is called after the GUI is
     * initialized.
     */
    private void initAfterGUI() {
        logger.entering();

        // Initialize security at the beginning.
        initSecurity();

        // load myProxy before loading components from registries.
        loadMyProxy();

        initMyLead();
        initRegistry();
        initGPEL();

        // This has to be after gpel initialization.
        loadDefaultGraph();

        // This has to be after loading a graph.
        initMonitor();
    }

    private void initSecurity() {
        // Initializes XSUL invokers with SSL without client authentication.
        try {
            XBayaSecurity.init();
        } catch (RuntimeException e) {
            getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
        }
    }

    private void loadMyProxy() {
        if (this.configuration.isLoadMyProxy()) {
            MyProxyDialog dialog = new MyProxyDialog(this);
            dialog.show(true); // blocking
        }
    }

    private void initMyLead() {
        MyLeadConfiguration myLeadConfiguration = this.myLead
                .getConfiguration();
        try {
            myLeadConfiguration.set(this.configuration.getMyLeadAgentURL(),
                    this.configuration.getMyLeadUser(), this.configuration
                            .getMyLeadProject());
        } catch (RuntimeException e) {
            getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
        } catch (Error e) {
            getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
        }
    }

    /**
     * Initializes registris.
     */
    private void initRegistry() {
        logger.entering();

        final ComponentSelector componentTreeViewer = this.gui
                .getComponentSelector();
        try {
            SystemComponentRegistry registry = new SystemComponentRegistry();
            // This does not take time, so we can do it in the same thread.
            ComponentTreeNode componentTree = registry.getComponentTree();
            componentTreeViewer.addComponentTree(0, componentTree);
        } catch (RuntimeException e) {
            // This should not happen
            getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
        }

        List<String> localRegistryPaths = this.configuration.getLocalRegistry();
        for (String path : localRegistryPaths) {
            try {
                LocalComponentRegistry registry = new LocalComponentRegistry(
                        path);
                // XXX This might take time, so it's better to move to another
                // thread.
                ComponentTreeNode componentTree = registry.getComponentTree();
                componentTreeViewer.addComponentTree(componentTree);
            } catch (ComponentRegistryException e) {
                getErrorWindow().error(ErrorMessages.COMPONENT_LIST_LOAD_ERROR,
                        e);
            } catch (RuntimeException e) {
                getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
            }
        }

        if (this.configuration.getResourceCatalogURL() != null) {
            new Thread() {
                @Override
                public void run() {
                    try {
                        ResourceCatalogClient client = new ResourceCatalogClient(
                                XBayaEngine.this.configuration
                                        .getResourceCatalogURL());
                        ComponentTreeNode componentTree = client
                                .getComponentTree();
                        componentTreeViewer.addComponentTree(componentTree);
                    } catch (ComponentRegistryException e) {
                        getErrorWindow().error(
                                ErrorMessages.COMPONENT_LIST_LOAD_ERROR, e);
                    } catch (RuntimeException e) {
                        getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                                e);
                    }
                }
            }.start();
        }

        if (this.configuration.getXRegistryURL() != null) {
            new Thread() {
                @Override
                public void run() {
                    try {
                        XRegistry client = new XRegistry(
                                XBayaEngine.this.configuration
                                        .getXRegistryURL(),
                                XRegistry.Type.ABSTRACT,
                                XBayaEngine.this.myProxyClient.getProxy());
                        ComponentTreeNode componentTree = client
                                .getComponentTree();
                        componentTreeViewer.addComponentTree(componentTree);
                    } catch (ComponentRegistryException e) {
                        getErrorWindow().error(
                                ErrorMessages.COMPONENT_LIST_LOAD_ERROR, e);
                    } catch (RuntimeException e) {
                        getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                                e);
                    }
                }
            }.start();
        }
    }

    /**
     * Loads a default graph if needed either from myLead or the server.
     */
    private void loadDefaultGraph() {
        URI gpelInstanceID = this.configuration.getGPELInstanceID();
        URI gpelTemplateID = this.configuration.getGPELTemplateID();
        String myLeadWorkflow = this.configuration.getMyLeadWorkflow();
        String localWorkflow = this.configuration.getWorkflow();

        if (gpelInstanceID != null) {
            GPELLoader loader = new GPELLoader(this);
            // Blocking call because the notification subscription have to occur
            // after this.
            loader.load(gpelInstanceID, WorkflowType.INSTANCE, true);
        } else if (gpelTemplateID != null) {
            GPELLoader loader = new GPELLoader(this);
            // Blocking call because the notification subscription have to occur
            // after this.
            loader.load(gpelTemplateID, WorkflowType.TEMPLATE, true);
        } else if (myLeadWorkflow != null) {
            MyLeadLoader loader = new MyLeadLoader(this);
            // Blocking call because the notification subscription have to occur
            // after this.
            loader.load(myLeadWorkflow, true);
        } else if (localWorkflow != null) {
            loadLocalGraph(localWorkflow);
        }
    }

    private void loadLocalGraph(String graphFilePath) {
        try {
            Workflow workflow = new Workflow(XMLUtil.loadXML(new File(
                    graphFilePath)));
            setWorkflow(workflow);
        } catch (GraphException e) {
            getErrorWindow().error(ErrorMessages.GRAPH_FORMAT_ERROR, e);
        } catch (ComponentException e) {
            getErrorWindow().error(ErrorMessages.GRAPH_FORMAT_ERROR, e);
        } catch (IOException e) {
            getErrorWindow().error(ErrorMessages.OPEN_FILE_ERROR, e);
        } catch (RuntimeException e) {
            getErrorWindow().error(ErrorMessages.GRAPH_LOAD_ERROR, e);
        } catch (Error e) {
            getErrorWindow().error(ErrorMessages.GRAPH_LOAD_ERROR, e);
        }
    }

    private void initGPEL() {
        URI engineURL = this.configuration.getGPELEngineURL();
        try {
            this.gpelClient.setEngineURL(engineURL);
        } catch (GPELException e) {
            getErrorWindow().error(ErrorMessages.GPEL_CONNECTION_ERROR, e);
        } catch (RuntimeException e) {
            getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
        } catch (Error e) {
            getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
        }
    }

    /**
     * Initializes monitor.
     */
    private void initMonitor() {
        try {
            if (this.configuration.isStartMonitor()) {
                MonitorStarter starter = new MonitorStarter(this);
                starter.start();
            }
        } catch (RuntimeException e) {
            getErrorWindow().error(ErrorMessages.MONITOR_SUBSCRIPTION_ERROR, e);
        } catch (Error e) {
            getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR, e);
        }
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
